package it.volga.ai;

import it.volga.model.Doom;
import it.volga.model.Location;
import it.volga.model.tiles.Portal;
import it.volga.model.tiles.Wall;
import lombok.val;

import java.util.HashMap;
import java.util.Map;

public abstract class AI
{
    final Doom model;
    final Graph graph;
    private final int desiredSize;

    public AI(Doom model)
    {
        this.model = model;
        this.desiredSize = numOfWalkableTiles();
        this.graph = buildGraph();
    }

    private Graph buildGraph()
    {
        val spawnAt = model.spawnAt();
        val result = new Graph();
        val spawn = model.tile(spawnAt);
        val root = result.addVertex(spawn, spawnAt);
        val history = new HashMap<Location, Vertex>(desiredSize);
        history.put(spawnAt, root);
        buildGraphRec(result, root, spawnAt, history);
        return result;
    }

    public Graph getGraph()
    {
        return graph;
    }

    private int numOfWalkableTiles()
    {
        int numberOfWalkableTiles = 0;
        for (int row = 0; row < model.rows(); row++)
        {
            for (int col = 0; col < model.cols(); col++)
            {
                if (canWalk(Location.of(row, col)))
                    numberOfWalkableTiles++;
            }
        }
        return numberOfWalkableTiles;
    }

    private void buildGraphRec(Graph graph, Vertex v, Location at, Map<Location, Vertex> history)
    {
        if (graph.order() != desiredSize)
        {
            for (Move m : Move.values())
            {
                val translated = m.translate(at);
                val tile = model.tile(translated);
                final boolean seen = history.containsKey(translated);
                if (canWalk(translated) && !seen)
                {
                    val neighbor = graph.addVertex(tile, translated);
                    graph.chain(v, neighbor);
                    history.put(translated, neighbor);
                    buildGraphRec(graph, neighbor, translated, history);
                }
                else if (seen)
                {
                    val seenVertex = history.get(translated);
                    if (!graph.connected(v, seenVertex))
                        graph.chain(v, seenVertex);
                }
                //
                if (v.tile() instanceof Portal)
                {
                    val p = ((Portal) v.tile()).getDual();
                    val loc = model.findTile(p);
                    if (!history.containsKey(loc))
                    {
                        buildGraphRec(graph, graph.addVertex(p, loc), loc, history);
                    }
                }
            }
        }
    }

    private boolean canWalk(Location at)
    {
        return !(model.tile(at) instanceof Wall);
    }

    public abstract Path solve() throws IntelligenceException;

}
