package it.volga.view.stats;

import it.volga.model.Rook;
import it.volga.view.Point;
import lombok.val;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;

public abstract class Statistics
{

    public static final int FONT_SIZE = 15;
    public static final int LETTER_HEIGHT = FONT_SIZE;
    public static final int LETTER_WIDTH = (int) (FONT_SIZE / 1.5);

    private final String label;
    private final Rook player;
    private Font boldFont;
    private final int length;
    private final int boxWidth;
    private final int boxHeight;
    private final Color background;

    protected Statistics(String label, Rook player, int statisticsMaxLength, Color background)
    {
        this.label = label;
        this.player = player;
        this.length = label.length() + statisticsMaxLength;
        this.boxHeight = LETTER_HEIGHT;
        this.boxWidth = LETTER_WIDTH * this.length;
        this.background = background;
    }

    public int length()
    {
        return length;
    }

    private void clear(Graphics2D g2d, Point upperLeftCornerOfBox)
    {
        g2d.setColor(background);
        g2d.fillRect(upperLeftCornerOfBox.axis(), upperLeftCornerOfBox.ordinate(), boxWidth, boxHeight);
    }

    public Point render(Graphics2D g2d, Point base)
    {
        val upperLeftCornerOfBox = Point.of(base.axis(), base.ordinate() - boxHeight);
        clear(g2d, upperLeftCornerOfBox);
        g2d.setFont(deriveFont(g2d));
        g2d.setColor(Color.BLACK);
        g2d.drawString(label + statistics(this.player), base.axis(), base.ordinate());
        return Point.of(base.axis() + boxWidth, base.ordinate());
    }

    abstract String statistics(Rook player);

    private Font deriveFont(Graphics2D g2d)
    {
        if (boldFont == null)
            boldFont = g2d.getFont().deriveFont(Font.BOLD, FONT_SIZE);
        return boldFont;
    }

}
