package it.volga.ai;

import it.volga.model.Location;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public enum Move
{

    RIGHT(0, 1, "LEFT"),
    LEFT(0, -1, "RIGHT"),
    DOWN(1, 0, "UP"),
    UP(-1, 0, "DOWN");

    public static Optional<Move> resolve(Location from, Location to)
    {
        final Location delta = Location.of(to.row() - from.row(), to.col() - from.col());
        for (Move move : Move.values())
            if (move.delta.equals(delta))
                return Optional.of(move);
        return Optional.empty();
    }

    public static List<Location> neighborhood(final Location center)
    {
        return Arrays.stream(Move.values())
                .map(move -> center.translate(move.delta))
                .collect(Collectors.toList());
    }

    private final Location delta;
    private final String reverseMoveName;

    Move(int dRow, int dCol, String reverseMoveName)
    {
        this.delta = Location.of(dRow, dCol);
        this.reverseMoveName = reverseMoveName;
    }

    public Location translate(final Location location)
    {
        return location.translate(this.delta);
    }

    public Location delta()
    {
        return delta;
    }

    public char letter()
    {
        return name().charAt(0);
    }

    public Move reverse()
    {
        return Move.valueOf(reverseMoveName);
    }

}
