package it.volga.controller;

import it.volga.ai.Move;
import it.volga.ai.Path;
import it.volga.model.Doom;
import it.volga.model.GameState;
import it.volga.view.View;

import javax.swing.JOptionPane;
import javax.swing.Timer;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;

public class AIController extends Controller implements ActionListener
{

    private final Move[] path;
    private final Timer animationTimer;
    private int i;

    public AIController(View view, Doom model, Path solution)
    {
        super(view, model);
        this.path = Arrays.stream(solution.nodes())
                .map(Path.Node::getMove)
                .toArray(Move[]::new);
        this.animationTimer = new Timer(300, this);
        this.animationTimer.setInitialDelay(1000);
    }

    public void walk()
    {
        boolean failedToStep = !step(path[i++]);
        if (failedToStep)
        {
            System.out.println("Не фортануло, братан");
            JOptionPane.showMessageDialog(null, "Не получилось пройти уровень",
                    "Ошибка", JOptionPane.ERROR_MESSAGE);
            this.animationTimer.stop();
        }
        else if (i == path.length || model.state() == GameState.WIN)
        {
            System.out.println("Успех!");
            JOptionPane.showMessageDialog(null, "Вы выйграли!",
                    "Успех", JOptionPane.INFORMATION_MESSAGE);
            this.animationTimer.stop();
        }
    }

    @Override
    public void actionPerformed(ActionEvent e)
    {
        walk();
    }

    @Override
    public void start()
    {
        super.start();
        System.out.println("Поехали!");
        animationTimer.start();
    }

}
