package it.volga.model.tiles;

import lombok.val;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

public class TileResolver
{

    private static final Map<Character, Tile> charToTile = new HashMap<>();

    static
    {
        val floor = new Floor(); charToTile.put(floor.symbol(), floor);
        val wall = new Wall(); charToTile.put(wall.symbol(), wall);
        val spawn = new Spawn(); charToTile.put(spawn.symbol(), spawn);
        val exit = new Exit(); charToTile.put(exit.symbol(), exit);
        val crate = new Crate(); charToTile.put(crate.symbol(), crate);
        val buttonA = new Button('a'); charToTile.put(buttonA.symbol(), buttonA);
        val buttonB = new Button('b'); charToTile.put(buttonB.symbol(), buttonB);
        val buttonC = new Button('c'); charToTile.put(buttonC.symbol(), buttonC);
        val buttonD = new Button('d'); charToTile.put(buttonD.symbol(), buttonD);
        val doorA = new Door('A', buttonA); charToTile.put(doorA.symbol(), doorA);
        val doorB = new Door('B', buttonB); charToTile.put(doorB.symbol(), doorB);
        val doorC = new Door('C', buttonC); charToTile.put(doorC.symbol(), doorC);
        val doorD = new Door('D', buttonD); charToTile.put(doorD.symbol(), doorD);
        val portalF = new Portal('F'); charToTile.put(portalF.symbol(), portalF);
        val portalf = new Portal('f'); charToTile.put(portalf.symbol(), portalf);
        portalF.setDual(portalf);
        portalf.setDual(portalF);
        val portalH = new Portal('H'); charToTile.put(portalH.symbol(), portalH);
        val portalh = new Portal('h'); charToTile.put(portalh.symbol(), portalh);
        portalH.setDual(portalh);
        portalh.setDual(portalH);
        val portalG = new Portal('G'); charToTile.put(portalG.symbol(), portalG);
        val portalg = new Portal('g'); charToTile.put(portalg.symbol(), portalg);
        portalG.setDual(portalg);
        portalg.setDual(portalG);
        val portalI = new Portal('I'); charToTile.put(portalI.symbol(), portalI);
        val portali = new Portal('i'); charToTile.put(portali.symbol(), portali);
        portalI.setDual(portali);
        portali.setDual(portalI);
        val portalJ = new Portal('J'); charToTile.put(portalJ.symbol(), portalJ);
        val portalj = new Portal('j'); charToTile.put(portalj.symbol(), portalj);
        portalJ.setDual(portalj);
        portalj.setDual(portalJ);
    }

    public static Optional<Tile> resolve(final char symbol)
    {
        return Optional.ofNullable(charToTile.getOrDefault(symbol, null));
    }

}
