package it.volga.view;

import it.volga.model.tiles.Tile;
import it.volga.model.tiles.TileResolver;
import lombok.val;

import javax.imageio.ImageIO;
import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Optional;

enum Sprite
{

    PLAYER("guy", '_'),
    FLOOR("floor", '.'),
    WALL("wall", 'X'),
    CRATE("crate", 'K'),
    SPAWN("floor", 'S'),
    DOOR_A("doorA", 'A'),
    DOOR_B("doorB", 'B'),
    DOOR_C("doorC", 'C'),
    DOOR_D("doorD", 'D'),
    BUTTON_A("buttonA", 'a'),
    BUTTON_B("buttonB", 'b'),
    BUTTON_C("buttonC", 'c'),
    BUTTON_D("buttonD", 'd'),
    PORTAL_BIG_F("portalBIGF", 'F'),
    PORTAL_BIG_G("portalBIGG", 'G'),
    PORTAL_BIG_H("portalBIGH", 'H'),
    PORTAL_BIG_I("portalBIGI", 'I'),
    PORTAL_BIG_J("portalBIGJ", 'J'),
    PORTAL_SMALL_F("portalsmallf", 'f'),
    PORTAL_SMALL_G("portalsmallg", 'g'),
    PORTAL_SMALL_H("portalsmallh", 'h'),
    PORTAL_SMALL_I("portalsmalli", 'i'),
    PORTAL_SMALL_J("portalsmallj", 'j'),
    EXIT("exit", 'Q');

    private static final String SPRITE_DIR = "src/main/resources/";

    private static boolean isRunningFromJar()
    {
        return Sprite.class.getResource("Sprite.class").toString().startsWith("jar");
    }

    public static Optional<Sprite> resolve(Tile tile)
    {
        for (val sprite : Sprite.values())
            if (sprite.tiles.contains(tile))
                return Optional.of(sprite);
        //
        return Optional.empty();
    }

    private Image picture;
    private final List<Tile> tiles;

    Sprite(String filename, char... symbols)
    {
        this.tiles = new LinkedList<>();
        this.picture = read(filename);
        for (char symbol : symbols)
            TileResolver.resolve(symbol).ifPresent(this.tiles::add);
    }

    private Image read(String filename)
    {
        final String full = "/" + filename + ".jpg";
        try
        {
            if (isRunningFromJar())
                return ImageIO.read(getClass().getResource(full));
            //
            val pngFile = new File(SPRITE_DIR + full);
            if (pngFile.exists())
                return ImageIO.read(pngFile);
            else
                throw new IllegalStateException("не могу найти файл " + full);
        }
        catch (IOException io)
        {
            throw new IllegalStateException(io);
        }
    }

    public void setPicture(Image image)
    {
        this.picture = image;
    }

    public Image picture()
    {
        return picture;
    }

}
