package it.volga.ai;

import it.volga.model.Doom;
import it.volga.model.GameState;
import lombok.val;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class AIPolynomialTest
{

    private static final String resourceDir = "src/main/resources/";

    private void solve(String mapName) throws Exception
    {
        val model = new Doom(resourceDir + mapName);
        val ai = new AIPolynomial(model);
        val solution = ai.solve();
        assertNotNull(solution);
        assertEquals(GameState.WIN, model.state());
    }

    private void fail(String mapName) throws Exception
    {
        val model = new Doom(resourceDir + mapName);
        val ai = new AIPolynomial(model);
        val solution = ai.solve();
        assertNotNull(solution);
        assertNotEquals(GameState.WIN, model.state());
    }

    private void noSolution(String mapName) throws Exception
    {
        val model = new Doom(resourceDir + mapName);
        val ai = new AIPolynomial(model);
        assertThrows(IntelligenceException.class, ai::solve);
    }

    @Test
    void solveMap1() throws Exception
    {
        solve("maps/map1.txt");
    }

    @Test
    void solveMap2() throws Exception
    {
        solve("map2.txt");
    }

    @Test
    void solveMap3() throws Exception
    {
        solve("map3.txt");
    }

    @Test
    void solveMap4() throws Exception
    {
        solve("map4.txt");
    }

    @Test
    void solveMap5() throws Exception
    {
        solve("map5.txt");
    }

    @Test
    void solveMap6() throws Exception
    {
        solve("map6.txt");
    }

    @Test
    void failMap7() throws Exception
    {
        solve("map7.txt");
    }

    @Test
    void solveMap8() throws Exception
    {
        solve("map8.txt");
    }

    @Test
    void solveMap9() throws Exception
    {
        solve("map9.txt");
    }

    @Test
    void solveMap10() throws Exception
    {
        noSolution("map10.txt");
    }

    @Test
    void solveMap11() throws Exception
    {
        solve("map11.txt");
    }

    @Test
    void solveMap12() throws Exception
    {
        solve("map12.txt");
    }

    @Test
    void solveMap13() throws Exception
    {
        solve("map13.txt");
    }

    @Test
    void solveMap14() throws Exception
    {
        solve("map14.txt");
    }

    @Test
    void solveMap15() throws Exception
    {
        solve("map15.txt");
    }

    @Test
    void solveMap16() throws Exception
    {
        solve("map16.txt");
    }

    @Test
    void solveMap17() throws Exception
    {
        solve("map17.txt");
    }

    @Test
    void solveMap18() throws Exception
    {
        noSolution("map18.txt");
    }


}